@@ -17,31 +17,53 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from offlineimap import imaplib, imaputil, imapserver, repository, folder, mbnames, threadutil
+from offlineimap import imaplib, imaputil, imapserver, repository, folder, mbnames, threadutil, version
 from offlineimap.threadutil import InstanceLimitedThread, ExitNotifyThread
 import re, os, os.path, offlineimap, sys
 from ConfigParser import ConfigParser
 from threading import *
+from getopt import getopt
 
-if '-d' in sys.argv:
+options = {}
+if '--help' in sys.argv[1:]:
+    sys.stdout.write(version.cmdhelp + "\n")
+    sys.exit(0)
+    
+for optlist in getopt(sys.argv[1:], '1a:c:du:h')[0]:
+    options[optlist[0]] = optlist[1]
+
+if '-d' in options:
     imaplib.Debug = 5
+if '-h' in options:
+    sys.stdout.write(version.cmdhelp)
+    sys.stdout.write("\n")
+    sys.exit(0)
+configfilename = os.path.expanduser("~/.offlineimaprc")
+if '-c' in options:
+    configfilename = options['-c']
 
-ui = offlineimap.ui.TTY.TTYUI()
-ui.init_banner()
 
 config = ConfigParser()
-configfilename = os.path.expanduser("~/.offlineimaprc")
 if not os.path.exists(configfilename):
     sys.stderr.write(" *** Config file %s does not exist; aborting!\n" % configfilename)
     sys.exit(1)
     
 config.read(configfilename)
 
+if '-u' in options:
+    ui = offlineimap.ui.detector.getUImod(options['-u'])()
+else:
+    ui = offlineimap.ui.detector.findUI(config)
+ui.init_banner()
+
+
 metadatadir = os.path.expanduser(config.get("general", "metadata"))
 if not os.path.exists(metadatadir):
     os.mkdir(metadatadir, 0700)
 
 accounts = config.get("general", "accounts")
+if '-a' in options:
+    accounts = options['-a']
 accounts = accounts.replace(" ", "")
 accounts = accounts.split(",")
 
@@ -51,8 +73,11 @@ localrepos = None
 passwords = {}
 tunnels = {}
 
-threadutil.initInstanceLimit("ACCOUNTLIMIT", config.getint("general",
-                                                           "maxsyncaccounts"))
+if '-1' in options:
+    threadutil.initInstanceLimit("ACCOUNTLIMIT", 1)
+else:
+    threadutil.initInstanceLimit("ACCOUNTLIMIT",
+                                 config.getint("general", "maxsyncaccounts"))
 
 # We have to gather passwords here -- don't want to have two threads
 # asking for passwords simultaneously.
@@ -69,8 +94,11 @@ for account in accounts:
     else:
         passwords[account] = ui.getpass(account, config)
     for instancename in ["FOLDER_" + account, "MSGCOPY_" + account]:
-        threadutil.initInstanceLimit(instancename,
-                                     config.getint(account, "maxconnections"))
+        if '-1' in options:
+            threadutil.initInstanceLimit(instancename, 1)
+        else:
+            threadutil.initInstanceLimit(instancename,
+                                         config.getint(account, "maxconnections"))
 
 mailboxes = []
 mailboxlock = Lock()
@@ -114,7 +142,7 @@ def syncaccount(accountname, *args):
             thread = InstanceLimitedThread(\
                 instancename = 'FOLDER_' + accountname,
                 target = syncfolder,
-                name = "syncfolder-%s-%s" % \
+                name = "Folder sync %s[%s]" % \
                 (accountname, remotefolder.getvisiblename()),
                 args = (accountname, remoterepos, remotefolder, localrepos,
                         statusrepos))
@@ -136,25 +164,38 @@ def syncfolder(accountname, remoterepos, remotefolder, localrepos,
     localfolder = localrepos.\
                   getfolder(remotefolder.getvisiblename().\
                             replace(remoterepos.getsep(), localrepos.getsep()))
-    if not localfolder.isuidvalidityok(remotefolder):
-        ui.validityproblem(remotefolder)
-        return
+    # Load local folder
     ui.syncingfolder(remoterepos, remotefolder, localrepos, localfolder)
     ui.loadmessagelist(localrepos, localfolder)
     localfolder.cachemessagelist()
     ui.messagelistloaded(localrepos, localfolder, len(localfolder.getmessagelist().keys()))
 
+    # Load status folder.
+    statusfolder = statusrepos.getfolder(remotefolder.getvisiblename().\
+                                         replace(remoterepos.getsep(),
+                                                 statusrepos.getsep()))
+    statusfolder.cachemessagelist()
+
+    
+    # If either the local or the status folder has messages and
+    # there is a UID validity problem, warn and abort.
+    # If there are no messages, UW IMAPd loses UIDVALIDITY.
+    # But we don't really need it if both local folders are empty.
+    # So, in that case, save it off.
+    if (len(localfolder.getmessagelist()) or \
+        len(statusfolder.getmessagelist())) and \
+        not localfolder.isuidvalidityok(remotefolder):
+        ui.validityproblem(remotefolder)
+        return
+    else:
+        localfolder.saveuidvalidity(remotefolder.getuidvalidity())
+
     # Load remote folder.
     ui.loadmessagelist(remoterepos, remotefolder)
     remotefolder.cachemessagelist()
     ui.messagelistloaded(remoterepos, remotefolder,
                          len(remotefolder.getmessagelist().keys()))
 
-    # Load status folder.
-    statusfolder = statusrepos.getfolder(remotefolder.getvisiblename().\
-                                         replace(remoterepos.getsep(),
-                                                 statusrepos.getsep()))
-    statusfolder.cachemessagelist()
 
     #
 
@@ -188,7 +229,7 @@ def syncitall():
     for accountname in accounts:
         thread = InstanceLimitedThread(instancename = 'ACCOUNTLIMIT',
                                        target = syncaccount,
-                                       name = "syncaccount-%s" % accountname,
+                                       name = "Account sync %s" % accountname,
                                        args = (accountname,))
         thread.setDaemon(1)
         thread.start()
@@ -213,6 +254,7 @@ def sync_with_timer():
                     event = Event()
                     kaevents[accountname] = event
                     thread = ExitNotifyThread(target = servers[accountname].keepalive,
+                                              name = "Keep alive " + accountname,
                                               args = (config.getint(accountname, 'keepalive'), event))
                     thread.setDaemon(1)
                     thread.start()
@@ -232,6 +274,11 @@ def sync_with_timer():
         
 def threadexited(thread):
     if thread.getExitCause() == 'EXCEPTION':
+        if isinstance(thread.getExitException(), SystemExit):
+            # Bring a SystemExit into the main thread.
+            # Do not send it back to UI layer right now.
+            # Maybe later send it to ui.terminate?
+            raise SystemExit
         ui.threadException(thread)      # Expected to terminate
         sys.exit(100)                   # Just in case...
         os._exit(100)
@@ -244,7 +291,8 @@ def threadexited(thread):
         ui.threadExited(thread)
 
 threadutil.initexitnotify()
-t = ExitNotifyThread(target=sync_with_timer, name='sync_with_timer')
+t = ExitNotifyThread(target=sync_with_timer,
+                     name='Sync Runner')
 t.setDaemon(1)
 t.start()
 try:
@@ -253,6 +301,3 @@ except SystemExit:
     raise
 except:
     ui.mainException()                  # Also expected to terminate.
-
-
-    
