@@ -20,6 +20,7 @@ from Base import BaseFolder
 from offlineimap import imaputil, imaplib
 import rfc822
 from StringIO import StringIO
+from copy import copy
 
 class IMAPFolder(BaseFolder):
     def __init__(self, imapserver, name, visiblename, accountname):
@@ -45,30 +46,25 @@ class IMAPFolder(BaseFolder):
 
     def getuidvalidity(self):
         imapobj = self.imapserver.acquireconnection()
-        x = None
-        readonlysave = imapobj.is_readonly
-        imapobj.is_readonly = 1
         try:
-            x = imapobj.status(self.getfullname(), '(UIDVALIDITY)')[1][0]
+            # Primes untagged_responses
+            imapobj.select(self.getfullname(), readonly = 1)
+            return long(imapobj.untagged_responses['UIDVALIDITY'][0])
         finally:
             self.imapserver.releaseconnection(imapobj)
-            imapobj.is_readonly = readonlysave
-        uidstring = imaputil.imapsplit(x)[1]
-        return long(imaputil.flagsplit(uidstring)[1])
     
     def cachemessagelist(self):
         imapobj = self.imapserver.acquireconnection()
+        self.messagelist = {}
+
         try:
-            self.messagelist = {}
-            response = imapobj.status(self.getfullname(), '(MESSAGES)')[1][0]
-            result = imaputil.imapsplit(response)[1]
-            maxmsgid = long(imaputil.flags2hash(result)['MESSAGES'])
-            if (maxmsgid < 1):
-                # No messages?  return.
+            # Primes untagged_responses
+            imapobj.select(self.getfullname(), readonly = 1)
+            maxmsgid = long(imapobj.untagged_responses['EXISTS'][0])
+            if maxmsgid < 1:
+                # No messages; return
                 return
 
-            # Needed for fetch below
-            imapobj.select(self.getfullname(), readonly = 1)
             # Now, get the flags and UIDs for these.
             response = imapobj.fetch('1:%d' % maxmsgid, '(FLAGS UID)')[1]
         finally:
@@ -98,6 +94,8 @@ class IMAPFolder(BaseFolder):
     def savemessage(self, uid, content, flags):
         imapobj = self.imapserver.acquireconnection()
         try:
+            imapobj.select(self.getfullname()) # Needed for search
+            
             # This backend always assigns a new uid, so the uid arg is ignored.
             # In order to get the new uid, we need to save off the message ID.
 
@@ -129,11 +127,19 @@ class IMAPFolder(BaseFolder):
         try:
             imapobj.select(self.getfullname())
             result = imapobj.uid('store', '%d' % uid, 'FLAGS',
-                                 imaputil.flagsmaildir2imap(flags))[1][0]
+                                 imaputil.flagsmaildir2imap(flags))
+            assert result[0] == 'OK', 'Error with store: ' + r[1]
         finally:
             self.imapserver.releaseconnection(imapobj)
-        flags = imaputil.flags2hash(imaputil.imapsplit(result)[1])['FLAGS']
-        self.messagelist[uid]['flags'] = imaputil.flagsimap2maildir(flags)
+        result = result[1][0]
+        if not result:
+            self.messagelist[uid]['flags'] = flags
+        else:
+            flags = imaputil.flags2hash(imaputil.imapsplit(result)[1])['FLAGS']
+            self.messagelist[uid]['flags'] = imaputil.flagsimap2maildir(flags)
+
+    def addmessageflags(self, uid, flags):
+        self.addmessagesflags([uid], flags)
 
     def addmessagesflags(self, uidlist, flags):
         imapobj = self.imapserver.acquireconnection()
@@ -142,16 +148,36 @@ class IMAPFolder(BaseFolder):
             r = imapobj.uid('store',
                             ','.join([str(uid) for uid in uidlist]),
                             '+FLAGS',
-                            imaputil.flagsmaildir2imap(flags))[1]
+                            imaputil.flagsmaildir2imap(flags))
+            assert r[0] == 'OK', 'Error with store: ' + r[1]
+            r = r[1]
         finally:
             self.imapserver.releaseconnection(imapobj)
-        resultcount = 0
+        # Some IMAP servers do not always return a result.  Therefore,
+        # only update the ones that it talks about, and manually fix
+        # the others.
+        needupdate = copy(uidlist)
         for result in r:
-            resultcount += 1
-            flags = imaputil.flags2hash(imaputil.imapsplit(result)[1])['FLAGS']
-            uid = long(imaputil.flags2hash(imaputil.imapsplit(result)[1])['UID'])
+            if result == None:
+                # Compensate for servers that don't return anything from
+                # STORE.
+                continue
+            attributehash = imaputil.flags2hash(imaputil.imapsplit(result)[1])
+            if not ('UID' in attributehash and 'FLAGS' in attributehash):
+                # Compensate for servers that don't return a UID attribute.
+                continue
+            flags = attributehash['FLAGS']
+            uid = long(attributehash['UID'])
             self.messagelist[uid]['flags'] = imaputil.flagsimap2maildir(flags)
-        assert resultcount == len(uidlist), "Got incorrect number of results back"
+            try:
+                needupdate.remove(uid)
+            except ValueError:          # Let it slide if it's not in the list
+                pass
+        for uid in needupdate:
+            for flag in flags:
+                if not flag in self.messagelist[uid]['flags']:
+                    self.messagelist[uid]['flags'].append(flag)
+                self.messagelist[uid]['flags'].sort()
 
     def deletemessage(self, uid):
         self.deletemessages([uid])
@@ -170,6 +196,6 @@ class IMAPFolder(BaseFolder):
         finally:
             self.imapserver.releaseconnection(imapobj)
         for uid in uidlist:
-            del(self.messagelist[uid])
+            del self.messagelist[uid]
         
         
